@@ -1,37 +1,41 @@
+# pyright: reportUnknownMemberType=false, reportUnknownParameterType=false
+
 from __future__ import annotations
 
 import asyncio
 from collections import defaultdict
-from collections.abc import Awaitable, Callable, Iterator
+from collections.abc import Awaitable, Callable, Iterable, Iterator
 from contextlib import contextmanager
 from datetime import datetime
 from functools import partial
 import importlib.resources
+import json
 import os
 from pathlib import Path
 import typing
-from typing import Any, TypeVar
+from typing import TYPE_CHECKING, Any, TypeVar
 
 import aiohttp
 import aiohttp.web
-from aiohttp_rpc import JsonRpcMethod
-from aiohttp_rpc import middlewares as rpc_middlewares
+from aiohttp_rpc import JsonRpcMethod, middlewares as rpc_middlewares
 from aiohttp_rpc.errors import InvalidParams, ServerError
 from aiohttp_rpc.server import WsJsonRpcServer
+from attrs import evolve, frozen
+from cattrs import GenConverter
+from cattrs.preconf.json import configure_converter
 import click
+from exceptiongroup import ExceptionGroup
 from loguru import logger
-from pydantic import BaseModel, ValidationError
 import sqlalchemy as sa
-import toga
+import toga  # pyright: ignore
 from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
 from yarl import URL
 
-from instawow import __version__, db, matchers, models
-from instawow import results as R
+from instawow import __version__, db, matchers, models, results as R
 from instawow.cataloguer import CatalogueEntry
-from instawow.common import ChangelogFormat, Flavour, Strategy
-from instawow.config import Config, GlobalConfig
-from instawow.github_auth import DeviceCodeResponse, get_codes, poll_for_access_token
+from instawow.common import Flavour, SourceMetadata, infer_flavour_from_path
+from instawow.config import Config, GlobalConfig, SecretStr, config_converter
+from instawow.github_auth import get_codes, poll_for_access_token
 from instawow.manager import (
     LocksType,
     Manager,
@@ -41,12 +45,13 @@ from instawow.manager import (
     is_outdated,
 )
 from instawow.resolvers import Defn
-from instawow.utils import evolve_model_obj
-from instawow.utils import run_in_thread as t
-from instawow.utils import uniq
+from instawow.utils import reveal_folder, run_in_thread as t, uniq
 
 from . import frontend
 
+if TYPE_CHECKING:
+    ExceptionGroup = ExceptionGroup[Exception]
+
 _T = TypeVar('_T')
 _P = ParamSpec('_P')
 _ManagerBoundCoroFn: TypeAlias = 'Callable[Concatenate[Manager, _P], Awaitable[_T]]'
@@ -57,12 +62,31 @@ _ManagerQueue: TypeAlias = (
 
 LOCALHOST = '127.0.0.1'
 
+_converter = GenConverter(
+    unstruct_collection_overrides={
+        # TODO: Replace with ``collections.abc.Set``
+        frozenset: sorted,
+    }
+)
+configure_converter(_converter)
+_converter.register_structure_hook(Path, lambda v, _: Path(v))
+_converter.register_unstructure_hook(Path, str)
+
 
 class _ConfigError(ServerError):
     code = -32001
     message = 'invalid configuration parameters'
 
 
+def _structure_excs(excs: Iterable[BaseException]):
+    return [
+        {'loc': [n], 'msg': str(e)}
+        for e in excs
+        for *_, n in (getattr(e, '__note__', '').rpartition(' '),)
+        if n.isidentifier()
+    ]
+
+
 @contextmanager
 def _reraise_validation_error(
     error_class: type[ServerError | InvalidParams] = ServerError,
@@ -70,16 +94,15 @@ def _reraise_validation_error(
 ) -> Iterator[None]:
     try:
         yield
-    except ValidationError as error:
-        errors = error.errors()
-        logger.info(f'invalid request: {(values, errors)}')
-        raise error_class(data=errors) from error
+    except ExceptionGroup as exc_group:
+        logger.info(f'invalid request: {(values, exc_group)}')
+        raise error_class(data=_structure_excs(exc_group.exceptions))
 
 
 @t
 def _read_config(profile: str) -> Config:
     with _reraise_validation_error(_ConfigError):
-        return Config.read(None, profile)
+        return Config.read(GlobalConfig.read(), profile)
 
 
 methods: list[tuple[str, type[BaseParams]]] = []
@@ -88,20 +111,30 @@ methods: list[tuple[str, type[BaseParams]]] = []
 def _register_method(method: str):
     def inner(param_class: type[BaseParams]):
         methods.append((method, param_class))
-        return param_class
+        return frozen(slots=False)(param_class)
 
     return inner
 
 
-class BaseParams(BaseModel):
+@frozen(slots=False)
+class _ProfileParamMixin:
+    profile: str
+
+
+@frozen(slots=False)
+class _DefnParamMixin:
+    defns: typing.List[Defn]
+
+
+class BaseParams:
     @classmethod
     def bind(
         cls, method: str, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> JsonRpcMethod:
-        async def respond(**kwargs: Any) -> BaseModel:
+        async def respond(**kwargs: Any):
             with _reraise_validation_error(InvalidParams, kwargs):
-                params = cls.parse_obj(kwargs)
-            return await params.respond(managers, app_window)
+                self = _converter.structure(kwargs, cls)
+            return await self.respond(managers, app_window)
 
         return JsonRpcMethod(respond, name=method)
 
@@ -111,14 +144,6 @@ class BaseParams(BaseModel):
         raise NotImplementedError
 
 
-class _ProfileParamMixin(BaseModel):
-    profile: str
-
-
-class _DefnParamMixin(BaseModel):
-    defns: typing.List[Defn]
-
-
 @_register_method('config/write_profile')
 class WriteProfileConfigParams(_ProfileParamMixin, BaseParams):
     addon_dir: Path
@@ -130,23 +155,24 @@ class WriteProfileConfigParams(_ProfileParamMixin, BaseParams):
     ) -> Config:
         async with managers.locks['modify profile', self.profile]:
             with _reraise_validation_error(_ConfigError):
-                config = Config(
-                    global_config=await t(GlobalConfig.read)(),
-                    profile=self.profile,
-                    addon_dir=self.addon_dir,
-                    game_flavour=self.game_flavour,
+                config = config_converter.structure(
+                    {
+                        'global_config': await t(GlobalConfig.read)(),
+                        'profile': self.profile,
+                        'addon_dir': self.addon_dir,
+                        'game_flavour': self.game_flavour,
+                    },
+                    Config,
                 )
                 if self.infer_game_flavour:
-                    values = {
-                        **config.dict(),
-                        'game_flavour': Config.infer_flavour(config.addon_dir),
-                    }
-                    config = Config(**values)
-
+                    config = evolve(
+                        config,
+                        game_flavour=infer_flavour_from_path(config.addon_dir),
+                    )
                 await t(config.write)()
 
-            # Unload the ``Manager`` instance corresponding to this profile for
-            # the config to be reloaded on the next request
+            # Unload the corresponding ``Manager`` instance for the
+            # config to be reloaded on the next request
             managers.unload(config.profile)
 
             return config
@@ -174,11 +200,10 @@ class DeleteProfileConfigParams(_ProfileParamMixin, BaseParams):
 
 @_register_method('config/list_profiles')
 class ListProfilesParams(BaseParams):
-    @t
-    def respond(
+    async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> list[str]:
-        return GlobalConfig().list_profiles()
+        return await t(GlobalConfig().list_profiles)()
 
 
 @_register_method('config/update_global')
@@ -190,26 +215,28 @@ class UpdateGlobalConfigParams(BaseParams):
     ) -> GlobalConfig:
         async with managers.locks['update global config']:
             with _reraise_validation_error(_ConfigError):
-                existing_global_config = await t(GlobalConfig.read)()
-                new_global_config = evolve_model_obj(
-                    existing_global_config,
-                    access_tokens=evolve_model_obj(
-                        existing_global_config.access_tokens, cfcore=self.cfcore_access_token
+                global_config = await t(GlobalConfig.read)()
+                global_config = evolve(
+                    global_config,
+                    access_tokens=evolve(
+                        global_config.access_tokens,
+                        cfcore=self.cfcore_access_token
+                        if self.cfcore_access_token is None
+                        else SecretStr(self.cfcore_access_token),
                     ),
                 )
-                await t(new_global_config.write)()
+                await t(global_config.write)()
 
         managers.unload_all()
-        return new_global_config
+        return global_config
 
 
 @_register_method('config/read_global')
 class ReadGlobalConfigParams(BaseParams):
-    @t
-    def respond(
+    async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> GlobalConfig:
-        return GlobalConfig.read()
+        return await t(GlobalConfig.read)()
 
 
 class GithubCodesResponse(TypedDict):
@@ -245,26 +272,13 @@ class CancelGithubAuthFlowParams(BaseParams):
         managers.cancel_github_auth_polling()
 
 
-class Source(TypedDict):
-    name: str
-    supported_strategies: list[Strategy]
-    changelog_format: ChangelogFormat
-
-
 @_register_method('sources/list')
 class ListSourcesParams(_ProfileParamMixin, BaseParams):
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
-    ) -> dict[str, Source]:
+    ) -> dict[str, SourceMetadata]:
         manager = await managers.get_manager(self.profile)
-        return {
-            r.source: {
-                'name': r.name,
-                'supported_strategies': sorted(r.strategies, key=list(Strategy).index),
-                'changelog_format': r.changelog_format,
-            }
-            for r in manager.resolvers.values()
-        }
+        return {r.metadata.id: r.metadata for r in manager.resolvers.values()}
 
 
 @_register_method('list')
@@ -323,7 +337,7 @@ class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
                 pair = manager.pair_uri(defn.alias)
                 if pair:
                     source, alias = pair
-                    defn = evolve_model_obj(defn, source=source, alias=alias)
+                    defn = evolve(defn, source=source, alias=alias)
             return defn
 
         return await manager.resolve(list(map(extract_source, self.defns)))
@@ -408,13 +422,15 @@ class PinParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
 
 @_register_method('get_changelog')
 class GetChangelogParams(_ProfileParamMixin, BaseParams):
+    source: str
     changelog_url: str
 
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> str:
         return await managers.run(
-            self.profile, partial(Manager.get_changelog, uri=self.changelog_url)
+            self.profile,
+            partial(Manager.get_changelog, source=self.source, uri=self.changelog_url),
         )
 
 
@@ -432,31 +448,38 @@ class AddonMatch_AddonFolder(TypedDict):
 class ReconcileParams(_ProfileParamMixin, BaseParams):
     matcher: Literal['toc_source_ids', 'folder_name_subsets', 'addon_names_with_folder_names']
 
+    _MATCHERS = {
+        'toc_source_ids': matchers.match_toc_source_ids,
+        'folder_name_subsets': matchers.match_folder_name_subsets,
+        'addon_names_with_folder_names': matchers.match_addon_names_with_folder_names,
+    }
+
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> list[AddonMatch]:
         leftovers = await managers.run(self.profile, t(matchers.get_unreconciled_folders))
-        match_groups: matchers.FolderAndDefnPairs = await managers.run(
-            self.profile, partial(getattr(matchers, f'match_{self.matcher}'), leftovers=leftovers)
+        match_groups = await managers.run(
+            self.profile, partial(self._MATCHERS[self.matcher], leftovers=leftovers)
         )
-        resolve_results = await managers.run(
+        resolved_defns = await managers.run(
             self.profile,
             partial(Manager.resolve, defns=uniq(d for _, b in match_groups for d in b)),
         )
-        matches = [
+        resolved_pkgs = {k: v for k, v in resolved_defns.items() if isinstance(v, models.Pkg)}
+        matched_pkgs = [
             (a, m)
             for a, s in match_groups
-            for m in ([r for d in s for r in (resolve_results[d],) if isinstance(r, models.Pkg)],)
+            for m in ([i for i in (resolved_pkgs.get(d) for d in s) if i],)
             if m
         ]
         return [
             *(
                 AddonMatch(folders=[{'name': f.name, 'version': f.version} for f in a], matches=m)
-                for a, m in matches
+                for a, m in matched_pkgs
             ),
             *(
                 AddonMatch(folders=[{'name': f.name, 'version': f.version}], matches=[])
-                for f in sorted(leftovers - frozenset(i for a, _ in matches for i in a))
+                for f in sorted(leftovers - frozenset(i for a, _ in matched_pkgs for i in a))
             ),
         ]
 
@@ -483,14 +506,15 @@ class GetReconcileInstalledCandidatesParams(_ProfileParamMixin, BaseParams):
         defn_groups = await managers.run(
             self.profile, partial(Manager.find_equivalent_pkg_defns, pkgs=installed_pkgs)
         )
-        resolve_results = await managers.run(
+        resolved_defns = await managers.run(
             self.profile,
             partial(Manager.resolve, defns=uniq(d for b in defn_groups.values() for d in b)),
         )
+        resolved_pkgs = {k: v for k, v in resolved_defns.items() if isinstance(v, models.Pkg)}
         return [
             {'installed_addon': p, 'alternative_addons': m}
             for p, s in defn_groups.items()
-            for m in ([r for d in s for r in (resolve_results[d],) if isinstance(r, models.Pkg)],)
+            for m in ([i for i in (resolved_pkgs.get(d) for d in s) if i],)
             if m
         ]
 
@@ -545,11 +569,11 @@ class RevealFolderParams(BaseParams):
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> None:
-        click.launch(os.path.join(*self.path_parts), locate=True)
+        reveal_folder(os.path.join(*self.path_parts))
 
 
 class SelectFolderResult(TypedDict):
-    selection: str | None
+    selection: Path | None
 
 
 @_register_method('assist/select_folder')
@@ -561,8 +585,10 @@ class SelectFolderParams(BaseParams):
     ) -> SelectFolderResult:
         if not app_window:
             raise RuntimeError('No app to bind to')
+
+        selection: Path | None
         try:
-            (selection,) = app_window.select_folder_dialog('Select folder', self.initial_folder)
+            selection = await app_window.select_folder_dialog('Select folder', self.initial_folder)
         except ValueError:
             selection = None
         return {'selection': selection}
@@ -582,7 +608,8 @@ class ConfirmDialogueParams(BaseParams):
     ) -> ConfirmDialogueResult:
         if not app_window:
             raise RuntimeError('No app to bind to')
-        ok = app_window.confirm_dialog(self.title, self.message)
+
+        ok: bool = await app_window.confirm_dialog(self.title, self.message)
         return {'ok': ok}
 
 
@@ -648,14 +675,14 @@ class _ManagerWorkQueue:
             close_db_conn()
 
     def unload_all(self):
-        for profile in self._managers:
+        for profile in list(self._managers):
             self.unload(profile)
 
     async def _schedule_item(
         self,
         future: asyncio.Future[object],
         profile: str,
-        coro_fn: _ManagerBoundCoroFn[_P, object],
+        coro_fn: _ManagerBoundCoroFn[..., object],
     ):
         try:
             async with self.locks['modify profile', profile]:
@@ -695,26 +722,34 @@ class _ManagerWorkQueue:
         return ((p, r()) for m, p, r in self._download_progress_reporters if m is manager)
 
     async def initiate_github_auth_flow(self):
-        async def _finalise_github_auth_flow():
-            result = await poll_for_access_token(
-                self._web_client, codes['device_code'], codes['interval']
-            )
+        async with self.locks['initiate github auth flow']:
+            if self._github_auth_device_codes is None:
 
-            async with self.locks['update global config']:
-                existing_global_config = await t(GlobalConfig.read)()
-                new_global_config = evolve_model_obj(
-                    existing_global_config,
-                    access_tokens=evolve_model_obj(
-                        existing_global_config.access_tokens, github=result
-                    ),
-                )
-                await t(new_global_config.write)()
+                async def finalise_github_auth_flow():
+                    result = await poll_for_access_token(
+                        self._web_client, codes['device_code'], codes['interval']
+                    )
 
-            self.unload_all()
+                    async with self.locks['update global config']:
+                        global_config = await t(GlobalConfig.read)()
+                        global_config = evolve(
+                            global_config,
+                            access_tokens=evolve(
+                                global_config.access_tokens, cfcore=SecretStr(result)
+                            ),
+                        )
+                        await t(global_config.write)()
+
+                    self.unload_all()
+
+                def on_task_complete(future: object):
+                    self._github_auth_flow_task = None
+                    self._github_auth_device_codes = None
+
+                self._github_auth_device_codes = codes = await get_codes(self._web_client)
+                self._github_auth_flow_task = self._loop.create_task(finalise_github_auth_flow())
+                self._github_auth_flow_task.add_done_callback(on_task_complete)
 
-        if self._github_auth_device_codes is None:
-            self._github_auth_device_codes = codes = await get_codes(self._web_client)
-            self._github_auth_flow_task = self._loop.create_task(_finalise_github_auth_flow())
         return self._github_auth_device_codes
 
     async def wait_for_github_auth_completion(self):
@@ -723,15 +758,11 @@ class _ManagerWorkQueue:
                 await self._github_auth_flow_task
             except BaseException:
                 return 'failure'
-            self._github_auth_flow_task = None
-        self._github_auth_device_codes = None
         return 'success'
 
     def cancel_github_auth_polling(self):
         if self._github_auth_flow_task is not None:
             self._github_auth_flow_task.cancel()
-            self._github_auth_flow_task = None
-        self._github_auth_device_codes = None
 
 
 async def create_app(app_window: toga.MainWindow | None = None):
@@ -762,7 +793,7 @@ async def create_app(app_window: toga.MainWindow | None = None):
         )
 
     def json_serialize(value: dict[str, Any]):
-        return BaseModel.construct(**value).json()
+        return json.dumps(_converter.unstructure(value))
 
     rpc_server = WsJsonRpcServer(
         json_serialize=json_serialize,
