@@ -1,5 +1,3 @@
-# pyright: reportUnknownMemberType=false
-
 from __future__ import annotations
 
 import asyncio
@@ -15,24 +13,22 @@ from collections.abc import (
 )
 from contextlib import contextmanager
 from datetime import datetime, timezone
-from functools import partial, wraps
+from functools import partial
 from itertools import chain, repeat
 from pathlib import Path
 import textwrap
 from typing import Any, NoReturn, TypeVar, overload
 
+from attrs import asdict, evolve, fields, resolve_types
 import click
 from loguru import logger
 
-from . import __version__, _deferred_types, db
-from . import manager as _manager
-from . import models
-from . import results as R
+from . import __version__, _deferred_types, db, manager as _manager, models, results as R
 from .common import Flavour, Strategy
-from .config import Config, GlobalConfig, setup_logging
+from .config import Config, GlobalConfig, config_converter, setup_logging
 from .plugins import load_plugins
 from .resolvers import ChangelogFormat, Defn
-from .utils import StrEnum, all_eq, cached_property, evolve_model_obj, gather, tabulate, uniq
+from .utils import StrEnum, all_eq, cached_property, gather, reveal_folder, tabulate, uniq
 
 _T = TypeVar('_T')
 _F = TypeVar('_F', bound='Callable[..., object]')
@@ -76,9 +72,9 @@ class Report:
         )
 
     def generate(self) -> None:
-        manager_wrapper: ManagerWrapper | None = click.get_current_context().obj
-        if manager_wrapper and manager_wrapper.manager.config.global_config.auto_update_check:
-            outdated, new_version = run_with_progress(_manager.is_outdated())
+        mw: _CtxObjWrapper | None = click.get_current_context().obj
+        if mw and mw.manager.config.global_config.auto_update_check:
+            outdated, new_version = mw.run_with_progress(_manager.is_outdated())
             if outdated:
                 click.echo(f'{self.WARNING_SYMBOL} instawow-{new_version} is available')
 
@@ -117,7 +113,7 @@ def _init_cli_web_client(
             )
             # The encoded size is not exposed in the aiohttp streaming API
             # so we cannot display progress when the payload is encoded.
-            # When ``None`` the progress bar is "indeterminate".
+            # When the total is ``None`` the progress bar is "indeterminate".
             total = None if hdrs.CONTENT_ENCODING in response.headers else response.content_length
 
             async def ticker():
@@ -127,7 +123,7 @@ def _init_cli_web_client(
                 progress_bar.counters.append(counter)
                 try:
                     while not response.content.is_eof():
-                        counter.items_completed = response.content.total_bytes
+                        counter.items_completed = response.content.total_bytes  # type: ignore
                         progress_bar.invalidate()
                         await asyncio.sleep(TICK_INTERVAL)
                 finally:
@@ -152,18 +148,6 @@ def _cancel_tickers(progress_bar: _deferred_types.prompt_toolkit.shortcuts.Progr
         progress_bar.invalidate()
 
 
-def run_with_progress(awaitable: Awaitable[_T]) -> _T:
-    from .prompts import make_progress_bar
-
-    async def run():
-        with make_progress_bar() as progress_bar, _cancel_tickers(progress_bar) as tickers:
-            async with _init_cli_web_client(progress_bar, tickers) as web_client:
-                _manager.contextualise(web_client=web_client)
-                return await awaitable
-
-    return asyncio.run(run())
-
-
 def _override_asyncio_loop_policy():
     # The proactor event loop which became the default loop on Windows
     # in Python 3.8 is causing issues with aiohttp.
@@ -177,30 +161,42 @@ def _apply_patches():
     _override_asyncio_loop_policy()
 
 
-class ManagerWrapper:
+class _CtxObjWrapper:
     def __init__(self, ctx: click.Context) -> None:
-        self.ctx = ctx
+        self._ctx = ctx
 
     @cached_property
     def manager(self) -> _manager.Manager:
+        global_config = GlobalConfig.read().ensure_dirs()
         try:
-            config = Config.read(None, self.ctx.params['profile']).ensure_dirs()
+            config = Config.read(global_config, self._ctx.params['profile']).ensure_dirs()
         except FileNotFoundError:
-            config = self.ctx.invoke(configure)
+            config = self._ctx.invoke(configure)
 
-        setup_logging(config.logging_dir, self.ctx.params['log_level'])
+        setup_logging(config.logging_dir, self._ctx.params['debug'])
         manager, close_db_conn = _manager.Manager.from_config(config)
-        self.ctx.call_on_close(close_db_conn)
+        self._ctx.call_on_close(close_db_conn)
 
         return manager
 
-    @staticmethod
-    def pass_manager(fn: _F) -> _F:
-        @wraps(fn)
-        def wrapper(*args: object, **kwargs: object):
-            return fn(click.get_current_context().obj.manager, *args, **kwargs)
+    def run_with_progress(self, awaitable: Awaitable[_T]) -> _T:
+        if self._ctx.params['debug']:
 
-        return wrapper  # type: ignore
+            async def run():
+                async with _manager.init_web_client() as web_client:
+                    _manager.contextualise(web_client=web_client)
+                    return await awaitable
+
+        else:
+            from .prompts import make_progress_bar
+
+            async def run():
+                with make_progress_bar() as progress_bar, _cancel_tickers(progress_bar) as tickers:
+                    async with _init_cli_web_client(progress_bar, tickers) as web_client:
+                        _manager.contextualise(web_client=web_client)
+                        return await awaitable
+
+        return asyncio.run(run())
 
 
 def _with_manager(fn: Callable[..., object]):
@@ -237,19 +233,16 @@ def _register_plugin_commands(group: click.Group):
     return group
 
 
-def _set_log_level(_: click.Context, __: click.Parameter, value: bool):
-    return 'DEBUG' if value else 'INFO'
-
-
 @_register_plugin_commands
-@click.group(context_settings={'help_option_names': ('-h', '--help')})
+@click.group(  # pyright: ignore[reportUnknownMemberType]
+    context_settings={'help_option_names': ('-h', '--help')}
+)
 @click.version_option(__version__, prog_name=__package__)
 @click.option(
     '--debug',
-    'log_level',
+    'debug',
     is_flag=True,
     default=False,
-    callback=_set_log_level,
     help='Log more things.',
 )
 @click.option(
@@ -259,10 +252,10 @@ def _set_log_level(_: click.Context, __: click.Parameter, value: bool):
     help='Activate the specified profile.',
 )
 @click.pass_context
-def cli(ctx: click.Context, log_level: str, profile: str) -> None:
+def cli(ctx: click.Context, **__: object) -> None:
     "Add-on manager for World of Warcraft."
     _apply_patches()
-    ctx.obj = ManagerWrapper(ctx)
+    ctx.obj = _CtxObjWrapper(ctx)
 
 
 main = logger.catch(reraise=True)(cli)
@@ -308,7 +301,7 @@ def parse_into_defn_with_strategy(
     manager: _manager.Manager, value: Sequence[tuple[Strategy, str]]
 ) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
-    return map(lambda d, s: evolve_model_obj(d, strategy=s), defns, (s for s, _ in value))
+    return map(lambda d, s: evolve(d, strategy=s), defns, (s for s, _ in value))
 
 
 def parse_into_defn_with_version(
@@ -357,22 +350,22 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
     help='A version followed by an add-on definition.',
 )
 @click.option('--replace', is_flag=True, default=False, help='Replace unreconciled add-ons.')
-@ManagerWrapper.pass_manager
-def install(manager: _manager.Manager, addons: Sequence[Defn], replace: bool) -> None:
+@click.pass_obj
+def install(mw: _CtxObjWrapper, addons: Sequence[Defn], replace: bool) -> None:
     "Install add-ons."
     if not addons:
         raise click.UsageError(
             'You must provide at least one of "ADDONS", "--with-strategy" or "--version"'
         )
 
-    results = run_with_progress(manager.install(addons, replace))
+    results = mw.run_with_progress(mw.manager.install(addons, replace))
     Report(results.items()).generate_and_exit()
 
 
 @cli.command()
 @click.argument('addons', nargs=-1, callback=_with_manager(parse_into_defn))
-@ManagerWrapper.pass_manager
-def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
+@click.pass_obj
+def update(mw: _CtxObjWrapper, addons: Sequence[Defn]) -> None:
     "Update installed add-ons."
     import sqlalchemy as sa
 
@@ -387,7 +380,7 @@ def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
 
     update_defns = addons or [
         Defn(**v)
-        for v in manager.database.execute(
+        for v in mw.manager.database.execute(
             sa.select(
                 db.pkg.c.source,
                 db.pkg.c.id,
@@ -399,7 +392,7 @@ def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
         .mappings()
         .all()
     ]
-    results = run_with_progress(manager.update(update_defns, False))
+    results = mw.run_with_progress(mw.manager.update(update_defns, False))
     Report(results.items(), filter_results).generate_and_exit()
 
 
@@ -411,10 +404,10 @@ def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
     default=False,
     help="Do not delete the add-on folders.",
 )
-@ManagerWrapper.pass_manager
-def remove(manager: _manager.Manager, addons: Sequence[Defn], keep_folders: bool) -> None:
+@click.pass_obj
+def remove(mw: _CtxObjWrapper, addons: Sequence[Defn], keep_folders: bool) -> None:
     "Remove add-ons."
-    results = run_with_progress(manager.remove(addons, keep_folders))
+    results = mw.run_with_progress(mw.manager.remove(addons, keep_folders))
     Report(results.items()).generate_and_exit()
 
 
@@ -430,24 +423,24 @@ def remove(manager: _manager.Manager, addons: Sequence[Defn], keep_folders: bool
     default=False,
     help='Undo rollback by reinstalling an add-on using the default strategy.',
 )
-@ManagerWrapper.pass_manager
-def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo: bool) -> None:
+@click.pass_obj
+def rollback(mw: _CtxObjWrapper, addon: Defn, version: str | None, undo: bool) -> None:
     "Roll an add-on back to an older version."
     from .prompts import Choice, ask, select
 
     if version and undo:
         raise click.UsageError('Cannot use "--version" with "--undo"')
 
-    pkg = manager.get_pkg(addon)
+    pkg = mw.manager.get_pkg(addon)
     if not pkg:
         report: Report = Report([(addon, R.PkgNotInstalled())])
         report.generate_and_exit()
 
-    if Strategy.version not in manager.resolvers[pkg.source].strategies:
+    if Strategy.version not in mw.manager.resolvers[pkg.source].metadata.strategies:
         Report([(addon, R.PkgStrategyUnsupported(Strategy.version))]).generate_and_exit()
 
     if undo:
-        Report(run_with_progress(manager.update([addon], True)).items()).generate_and_exit()
+        Report(mw.run_with_progress(mw.manager.update([addon], True)).items()).generate_and_exit()
 
     reconstructed_defn = Defn.from_pkg(pkg)
     if version:
@@ -472,8 +465,8 @@ def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo:
         )
 
     Report(
-        run_with_progress(
-            manager.update([reconstructed_defn.with_version(selection)], True)
+        mw.run_with_progress(
+            mw.manager.update([reconstructed_defn.with_version(selection)], True)
         ).items()
     ).generate_and_exit()
 
@@ -490,10 +483,8 @@ def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo:
 @click.option(
     '--list-unreconciled', is_flag=True, default=False, help='List unreconciled add-ons and exit.'
 )
-@ManagerWrapper.pass_manager
-def reconcile(
-    manager: _manager.Manager, auto: bool, rereconcile: bool, list_unreconciled: bool
-) -> None:
+@click.pass_obj
+def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconciled: bool) -> None:
     "Reconcile pre-installed add-ons."
     from .matchers import (
         AddonFolder,
@@ -520,12 +511,14 @@ def reconcile(
         groups: Sequence[tuple[Any, Sequence[Defn]]],
         selector: Callable[[Any, Sequence[models.Pkg]], Defn | None],
     ):
-        results = run_with_progress(manager.resolve(uniq(d for _, b in groups for d in b)))
+        results = mw.run_with_progress(mw.manager.resolve(uniq(d for _, b in groups for d in b)))
         for addons_or_pkg, defns in groups:
             shortlist = [r for d in defns for r in (results[d],) if isinstance(r, models.Pkg)]
             if shortlist:
                 selection = selector(addons_or_pkg, shortlist)
                 yield selection
+            else:
+                yield None
 
     if rereconcile:
         if auto:
@@ -544,22 +537,24 @@ def reconcile(
             return selection or None
 
         installed_pkgs = (
-            models.Pkg.from_row_mapping(manager.database, p)
-            for p in manager.database.execute(sa.select(db.pkg)).mappings().all()
+            models.Pkg.from_row_mapping(mw.manager.database, p)
+            for p in mw.manager.database.execute(sa.select(db.pkg)).mappings().all()
+        )
+        groups = list(
+            mw.run_with_progress(mw.manager.find_equivalent_pkg_defns(installed_pkgs)).items()
         )
-        groups = list(run_with_progress(manager.find_equivalent_pkg_defns(installed_pkgs)).items())
         selections = [
             (p, s) for (p, _), s in zip(groups, gather_selections(groups, prompt_reconciled)) if s
         ]
         if selections and ask(confirm('Install selected add-ons?')):
             Report(
-                run_with_progress(
-                    manager.remove([Defn.from_pkg(p) for p, _ in selections], False),
+                mw.run_with_progress(
+                    mw.manager.remove([Defn.from_pkg(p) for p, _ in selections], False),
                 ).items()
             ).generate()
             Report(
-                run_with_progress(
-                    manager.install([s for _, s in selections], False),
+                mw.run_with_progress(
+                    mw.manager.install([s for _, s in selections], False),
                 ).items()
             ).generate_and_exit()
 
@@ -609,10 +604,10 @@ def reconcile(
             match_folder_name_subsets,
             match_addon_names_with_folder_names,
         ]:
-            groups = run_with_progress(fn(manager, (yield [])))
+            groups = mw.run_with_progress(fn(mw.manager, (yield [])))
             yield list(filter(None, gather_selections(groups, selector)))
 
-    leftovers = get_unreconciled_folders(manager)
+    leftovers = get_unreconciled_folders(mw.manager)
     if list_unreconciled:
         table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
         click.echo(tabulate(table_rows))
@@ -628,10 +623,10 @@ def reconcile(
     for _ in matcher:  # Skip over consumer yields
         selections = matcher.send(leftovers)
         if selections and (auto or ask(confirm('Install selected add-ons?'))):
-            results = run_with_progress(manager.install(selections, True))
+            results = mw.run_with_progress(mw.manager.install(selections, True))
             Report(results.items()).generate()
 
-        leftovers = get_unreconciled_folders(manager)
+        leftovers = get_unreconciled_folders(mw.manager)
 
     if leftovers:
         click.echo()
@@ -680,15 +675,15 @@ def search(
     "Search for add-ons to install."
     from .prompts import PkgChoice, ask, checkbox, confirm
 
-    manager: _manager.Manager = ctx.obj.manager
+    mw: _CtxObjWrapper = ctx.obj
 
-    entries = run_with_progress(
-        manager.search(search_terms, limit, frozenset(sources), start_date)
+    entries = mw.run_with_progress(
+        mw.manager.search(search_terms, limit, frozenset(sources), start_date)
     )
     defns = [Defn(e.source, e.id) for e in entries]
     pkgs = (
-        (evolve_model_obj(d, alias=r.slug), r)
-        for d, r in run_with_progress(manager.resolve(defns)).items()
+        (evolve(d, alias=r.slug), r)
+        for d, r in mw.run_with_progress(mw.manager.resolve(defns)).items()
         if isinstance(r, models.Pkg)
     )
     choices = [PkgChoice(f'{p.name}  ({d.to_urn()}=={p.version})', d, pkg=p) for d, p in pkgs]
@@ -719,9 +714,9 @@ class _ListFormats(StrEnum):
     show_default=True,
     help='Change the output format.',
 )
-@ManagerWrapper.pass_manager
+@click.pass_obj
 def list_installed(
-    manager: _manager.Manager, addons: Sequence[Defn], output_format: _ListFormats
+    mw: _CtxObjWrapper, addons: Sequence[Defn], output_format: _ListFormats
 ) -> None:
     "List installed add-ons."
     import sqlalchemy as sa
@@ -731,17 +726,17 @@ def list_installed(
             Defn(pkg.source, s or e.id).to_urn()
             for e in pkg.deps
             for s in (
-                manager.database.execute(
+                mw.manager.database.execute(
                     sa.select(db.pkg.c.slug).filter_by(source=pkg.source, id=e.id)
                 ).scalar_one_or_none(),
             )
         )
 
     def row_mappings_to_pkgs():
-        return map(models.Pkg.from_row_mapping, repeat(manager.database), pkg_mappings)
+        return map(models.Pkg.from_row_mapping, repeat(mw.manager.database), pkg_mappings)
 
     pkg_mappings = (
-        manager.database.execute(
+        mw.manager.database.execute(
             sa.select(db.pkg)
             .filter(
                 sa.or_(
@@ -763,7 +758,17 @@ def list_installed(
     )
 
     if output_format is _ListFormats.json:
-        click.echo(models.PkgList.parse_obj(list(row_mappings_to_pkgs())).json(indent=2))
+        import json
+        import typing
+
+        click.echo(
+            json.dumps(
+                models.pkg_converter.unstructure(  # pyright: ignore[reportUnknownMemberType]
+                    row_mappings_to_pkgs(), typing.List[models.Pkg]
+                ),
+                indent=2,
+            )
+        )
 
     elif output_format is _ListFormats.detailed:
         formatter = click.HelpFormatter(max_width=99)
@@ -800,12 +805,12 @@ def info(ctx: click.Context, addon: Defn) -> None:
 
 @cli.command()
 @click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
-@ManagerWrapper.pass_manager
-def reveal(manager: _manager.Manager, addon: Defn) -> None:
+@click.pass_obj
+def reveal(mw: _CtxObjWrapper, addon: Defn) -> None:
     "Bring an add-on up in your file manager."
-    pkg = manager.get_pkg(addon, partial_match=True)
+    pkg = mw.manager.get_pkg(addon, partial_match=True)
     if pkg:
-        click.launch(str(manager.config.addon_dir / pkg.folders[0].name), locate=True)
+        reveal_folder(mw.manager.config.addon_dir / pkg.folders[0].name)
     else:
         Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
 
@@ -820,8 +825,8 @@ def reveal(manager: _manager.Manager, addon: Defn) -> None:
     show_default=True,
     help='Convert HTML and Markdown changelogs to plain text using pandoc.',
 )
-@ManagerWrapper.pass_manager
-def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool) -> None:
+@click.pass_obj
+def view_changelog(mw: _CtxObjWrapper, addon: Defn | None, convert: bool) -> None:
     """View the changelog of an installed add-on.
 
     If `addon` is not provided, displays the changelogs of all add-ons
@@ -843,7 +848,7 @@ def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool)
             def real_convert(source: str, changelog: str):
                 import subprocess
 
-                changelog_format = manager.resolvers[source].changelog_format
+                changelog_format = mw.manager.resolvers[source].metadata.changelog_format
                 if changelog_format not in {ChangelogFormat.html, ChangelogFormat.markdown}:
                     return changelog
                 else:
@@ -854,9 +859,11 @@ def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool)
             return real_convert
 
     if addon:
-        pkg = manager.get_pkg(addon, partial_match=True)
+        pkg = mw.manager.get_pkg(addon, partial_match=True)
         if pkg:
-            changelog = run_with_progress(manager.get_changelog(pkg.changelog_url))
+            changelog = mw.run_with_progress(
+                mw.manager.get_changelog(pkg.source, pkg.changelog_url)
+            )
             if convert:
                 changelog = make_converter()(pkg.source, changelog)
             click.echo_via_pager(changelog)
@@ -866,20 +873,27 @@ def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool)
     else:
         import sqlalchemy as sa
 
-        last_installed_changelog_urls = manager.database.execute(
+        join_clause = (db.pkg.c.source == db.pkg_version_log.c.pkg_source) & (
+            db.pkg.c.id == db.pkg_version_log.c.pkg_id
+        )
+
+        last_installed_changelog_urls = mw.manager.database.execute(
             sa.select(db.pkg.c.source, db.pkg.c.slug, db.pkg.c.changelog_url)
-            .join(db.pkg_version_log)
+            .join(db.pkg_version_log, join_clause)
             .filter(
                 db.pkg_version_log.c.install_time
                 >= sa.select(
                     sa.func.datetime(sa.func.max(db.pkg_version_log.c.install_time), '-1 minute')
                 )
-                .join(db.pkg)
+                .join(db.pkg, join_clause)
                 .scalar_subquery()
             )
         ).all()
-        changelogs = run_with_progress(
-            gather(manager.get_changelog(m.changelog_url) for m in last_installed_changelog_urls)
+        changelogs = mw.run_with_progress(
+            gather(
+                mw.manager.get_changelog(m.source, m.changelog_url)
+                for m in last_installed_changelog_urls
+            )
         )
         if convert:
             do_convert = make_converter()
@@ -896,7 +910,7 @@ def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool)
 
 def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool):
     if value:
-        click.echo(ctx.obj.manager.config.json(indent=2))
+        click.echo(ctx.obj.manager.config.encode_for_display())
         ctx.exit()
 
 
@@ -919,6 +933,7 @@ class _EditableConfigOptions(StrEnum):
     game_flavour = 'game_flavour'
     auto_update_check = 'auto_update_check'
     github_access_token = 'access_tokens.github'
+    cfcore_access_token = 'access_tokens.cfcore'
 
 
 @cli.command()
@@ -942,7 +957,8 @@ def configure(
     config_options: Collection[_EditableConfigOptions],
 ) -> Config:
     "Configure instawow."
-    from .prompts import PydanticValidator, ask, confirm, path, select
+    from .common import infer_flavour_from_path
+    from .prompts import AttrFieldValidator, ask, confirm, password, path, select
 
     if not config_options:
         config_options = {
@@ -954,11 +970,10 @@ def configure(
     profile = ctx.find_root().params['profile']
 
     orig_global_config = GlobalConfig.read()
-    global_config_values = orig_global_config.dict()
+    global_config_values = asdict(orig_global_config)
     try:
-        profile_config_values = Config.read(orig_global_config, profile).dict(
-            exclude={'global_config'}
-        )
+        profile_config_values = asdict(Config.read(orig_global_config, profile))
+        del profile_config_values['global_config']
     except FileNotFoundError:
         profile_config_values = {
             'profile': profile,
@@ -970,7 +985,9 @@ def configure(
             path(
                 'Add-on directory:',
                 only_directories=True,
-                validate=PydanticValidator(Config, 'addon_dir'),
+                validate=AttrFieldValidator(
+                    fields(resolve_types(Config)).addon_dir, config_converter
+                ),
             )
         )
         profile_config_values['addon_dir'] = addon_dir
@@ -980,7 +997,9 @@ def configure(
             select(
                 'Game flavour:',
                 choices=list(Flavour),
-                initial_choice=Config.infer_flavour(addon_dir) if addon_dir is not None else None,
+                initial_choice=infer_flavour_from_path(addon_dir)
+                if addon_dir is not None
+                else None,
             )
         )
         profile_config_values['game_flavour'] = game_flavour
@@ -996,8 +1015,15 @@ def configure(
         github_access_token = asyncio.run(_github_oauth_flow())
         global_config_values['access_tokens']['github'] = github_access_token
 
-    global_config = GlobalConfig(**global_config_values).write()
-    config = Config(global_config=global_config, **profile_config_values).write()
+    if _EditableConfigOptions.cfcore_access_token in config_options:
+        global_config_values['access_tokens']['cfcore'] = ask(
+            password('Enter CFCore access token:')
+        )
+
+    global_config = config_converter.structure(global_config_values, GlobalConfig).write()
+    config = config_converter.structure(
+        {**profile_config_values, 'global_config': global_config}, Config
+    ).write()
 
     click.echo('Configuration written to:')
     click.echo(f'  {global_config.config_file}')
@@ -1012,25 +1038,25 @@ def _weakauras_group() -> None:
 
 
 @_weakauras_group.command('build')
-@ManagerWrapper.pass_manager
-def build_weakauras_companion(manager: _manager.Manager) -> None:
+@click.pass_obj
+def build_weakauras_companion(mw: _CtxObjWrapper) -> None:
     "Build the WeakAuras Companion add-on."
     from .wa_updater import WaCompanionBuilder
 
-    run_with_progress(WaCompanionBuilder(manager).build())
+    mw.run_with_progress(WaCompanionBuilder(mw.manager).build())
 
 
 @_weakauras_group.command('list')
-@ManagerWrapper.pass_manager
-def list_installed_wago_auras(manager: _manager.Manager) -> None:
+@click.pass_obj
+def list_installed_wago_auras(mw: _CtxObjWrapper) -> None:
     "List WeakAuras installed from Wago."
     from .wa_updater import WaCompanionBuilder
 
-    aura_groups = WaCompanionBuilder(manager).extract_installed_auras()
+    aura_groups = WaCompanionBuilder(mw.manager).extract_installed_auras()
     installed_auras = sorted(
         (g.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
-        for v in g.__root__.values()
+        for v in g.root.values()
         for a in v
         if not a.parent
     )
@@ -1046,16 +1072,21 @@ def list_installed_wago_auras(manager: _manager.Manager) -> None:
 )
 def generate_catalogue(start_date: datetime | None) -> None:
     "Generate the master catalogue."
-    from .cataloguer import BaseCatalogue
+    import json
+
+    from .cataloguer import BaseCatalogue, catalogue_converter
 
     catalogue = asyncio.run(BaseCatalogue.collate(start_date))
+    catalogue_json = catalogue_converter.unstructure(  # pyright: ignore[reportUnknownMemberType]
+        catalogue
+    )
     catalogue_path = Path(f'base-catalogue-v{catalogue.version}.json').resolve()
     catalogue_path.write_text(
-        catalogue.json(indent=2),
+        json.dumps(catalogue_json, indent=2),
         encoding='utf-8',
     )
     catalogue_path.with_suffix(f'.compact{catalogue_path.suffix}').write_text(
-        catalogue.json(separators=(',', ':')),
+        json.dumps(catalogue_json, separators=(',', ':')),
         encoding='utf-8',
     )
 
@@ -1067,10 +1098,10 @@ def gui(ctx: click.Context) -> None:
     from instawow_gui.app import InstawowApp
 
     global_config = GlobalConfig.read().ensure_dirs()
-    dummy_jsonrpc_config = Config.construct(
+    dummy_jsonrpc_config = Config.make_dummy_config(
         global_config=global_config, profile='__jsonrpc__'
     ).ensure_dirs()
     params = ctx.find_root().params
-    setup_logging(dummy_jsonrpc_config.logging_dir, params['log_level'])
+    setup_logging(dummy_jsonrpc_config.logging_dir, params['debug'])
 
-    InstawowApp(version=__version__).main_loop()
+    InstawowApp(version=__version__).main_loop()  # pyright: ignore[reportUnknownMemberType]
