@@ -1,7 +1,9 @@
 from __future__ import annotations
 
+from contextlib import contextmanager
 from datetime import datetime, timezone
 from enum import IntEnum
+import sqlite3
 
 from sqlalchemy import (
     Column,
@@ -11,11 +13,13 @@ from sqlalchemy import (
     String,
     Table,
     TypeDecorator,
+    create_engine,
     func,
     text,
 )
+from sqlalchemy.event import listen
 from sqlalchemy.exc import OperationalError
-from sqlalchemy.future import Engine
+from sqlalchemy.future import Connection, Engine
 
 
 class TZDateTime(TypeDecorator[datetime]):
@@ -67,6 +71,7 @@ pkg_options = Table(
         ['pkg_source', 'pkg_id'],
         ['pkg.source', 'pkg.id'],
         name='fk_pkg_options_pkg_source_and_id',
+        ondelete='CASCADE',
     ),
 )
 
@@ -80,6 +85,7 @@ pkg_folder = Table(
         ['pkg_source', 'pkg_id'],
         ['pkg.source', 'pkg.id'],
         name='fk_pkg_folder_pkg_source_and_id',
+        ondelete='CASCADE',
     ),
 )
 
@@ -93,6 +99,7 @@ pkg_dep = Table(
         ['pkg_source', 'pkg_id'],
         ['pkg.source', 'pkg.id'],
         name='fk_pkg_dep_pkg_source_and_id',
+        ondelete='CASCADE',
     ),
 )
 
@@ -103,21 +110,16 @@ pkg_version_log = Table(
     Column('install_time', TZDateTime, nullable=False, server_default=func.now()),
     Column('pkg_source', String, primary_key=True),
     Column('pkg_id', String, primary_key=True),
-    ForeignKeyConstraint(
-        ['pkg_source', 'pkg_id'],
-        ['pkg.source', 'pkg.id'],
-        name='fk_pkg_version_log_pkg_source_and_id',
-    ),
 )
 
 
-class DatabaseState(IntEnum):
+class _DatabaseState(IntEnum):
     current = 0
     old = 1
     uninitialised = 2
 
 
-def get_database_state(engine: Engine, revision: str) -> DatabaseState:
+def _get_database_state(engine: Engine, revision: str) -> _DatabaseState:
     with engine.connect() as connection:
         try:
             state = connection.execute(
@@ -133,12 +135,29 @@ def get_database_state(engine: Engine, revision: str) -> DatabaseState:
                 )
             ).scalar()
 
-    return DatabaseState(state)
+    return _DatabaseState(state)
 
 
-def migrate_database(engine: Engine, revision: str) -> None:
-    database_state = get_database_state(engine, revision)
-    if database_state != DatabaseState.current:
+def _set_sqlite_pragma(dbapi_connection: sqlite3.Connection, connection_record: object):
+    cursor = dbapi_connection.cursor()
+    cursor.execute('PRAGMA foreign_keys = ON')
+    cursor.close()
+
+
+def prepare_database(uri: str, revision: str) -> Engine:
+    "Connect to and optionally create or migrate the database from a configuration object."
+    engine = create_engine(
+        uri,
+        # We wanna be able to operate on the database from executor threads
+        # when performing disk I/O
+        connect_args={'check_same_thread': False},
+        # echo=True,
+        future=True,
+    )
+    listen(engine, 'connect', _set_sqlite_pragma)
+
+    database_state = _get_database_state(engine, revision)
+    if database_state != _DatabaseState.current:
         import alembic.command
         import alembic.config
 
@@ -146,8 +165,21 @@ def migrate_database(engine: Engine, revision: str) -> None:
         alembic_config.set_main_option('script_location', f'{__package__}:migrations')
         alembic_config.set_main_option('sqlalchemy.url', str(engine.url))
 
-        if database_state == DatabaseState.uninitialised:
+        if database_state == _DatabaseState.uninitialised:
             metadata.create_all(engine)
             alembic.command.stamp(alembic_config, revision)
         else:
             alembic.command.upgrade(alembic_config, revision)
+
+    return engine
+
+
+@contextmanager
+def faux_transact(connection: Connection):
+    try:
+        yield
+    except BaseException:
+        connection.rollback()
+        raise
+    else:
+        connection.commit()
