@@ -14,24 +14,22 @@ from shutil import copy
 from tempfile import NamedTemporaryFile
 import time
 from typing import Any, TypeVar
-import urllib.parse
 
+from attrs import evolve, fields
 from loguru import logger
 import sqlalchemy as sa
 import sqlalchemy.future as sa_future
 from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
 from yarl import URL
 
-from . import _deferred_types, db, models
-from . import results as R
-from .cataloguer import BaseCatalogue, Catalogue, CatalogueEntry
+from . import _deferred_types, db, models, results as R
+from .cataloguer import BaseCatalogue, Catalogue, CatalogueEntry, catalogue_converter
 from .common import Strategy
 from .config import Config, GlobalConfig
+from .models import PkgFolder
 from .plugins import load_plugins
 from .resolvers import (
-    BaseResolver,
     CfCoreResolver,
-    CurseResolver,
     Defn,
     GithubResolver,
     InstawowResolver,
@@ -42,17 +40,18 @@ from .resolvers import (
 from .utils import (
     bucketise,
     chain_dict,
-    evolve_model_obj,
     file_uri_to_path,
-    find_addon_zip_base_dirs,
+    find_addon_zip_tocs,
     gather,
     is_not_stale,
     make_zip_member_filter,
     move,
     normalise_names,
+    run_in_thread as t,
+    shasum,
+    trash,
+    uniq,
 )
-from .utils import run_in_thread as t
-from .utils import shasum, trash, uniq
 
 _P = ParamSpec('_P')
 _T = TypeVar('_T')
@@ -60,7 +59,7 @@ _T = TypeVar('_T')
 
 USER_AGENT = 'instawow (https://github.com/layday/instawow)'
 
-DB_REVISION = '75f69831f74f'
+DB_REVISION = 'e13430219249'
 
 
 class _GenericDownloadTraceRequestCtx(TypedDict):
@@ -105,28 +104,33 @@ def _open_pkg_archive(path: PurePath):
             archive.extractall(parent, members=filter(make_zip_member_filter(base_dirs), names))
 
         names = archive.namelist()
-        base_dirs = set(find_addon_zip_base_dirs(names))
+        base_dirs = {h for _, h in find_addon_zip_tocs(names)}
         yield (base_dirs, extract)
 
 
 class _ResponseWrapper:
-    def __init__(self, response_obj: _deferred_types.aiohttp.ClientResponse | None, text: str):
-        self._response_obj = response_obj
-        self._text = text
+    def __init__(
+        self, response: _deferred_types.aiohttp.ClientResponse | None, response_body: bytes
+    ):
+        self._response = response
+        self._response_body = response_body
+
+    async def read(self) -> bytes:
+        return self._response_body
 
     async def text(self) -> str:
-        return self._text
+        return self._response_body.decode()
 
     async def json(self, **kwargs: Any) -> Any:
-        return json.loads(await self.text())
+        return json.loads(await self.read())
 
     def raise_for_status(self) -> None:
-        if self._response_obj is not None:
-            self._response_obj.raise_for_status()
+        if self._response is not None:
+            self._response.raise_for_status()
 
     @property
     def status(self) -> int:
-        return self._response_obj.status if self._response_obj is not None else 200
+        return self._response.status if self._response is not None else 200
 
 
 class _CacheFauxClientSession:
@@ -156,22 +160,22 @@ class _CacheFauxClientSession:
 
         async def make_request():
             async with self.wrapped.request(**prepare_request_kwargs()) as response:
-                return (response, await response.text())
+                return (response, await response.read())
 
         if ttl is None:
-            response, text = await make_request()
+            response, response_body = await make_request()
         else:
             dest = self._cache_dir / shasum(url, ttl, kwargs)
             response = None
             if await t(is_not_stale)(dest, ttl):
                 logger.debug(f'{url} is cached at {dest} (ttl: {ttl})')
-                text = await t(dest.read_text)(encoding='utf-8')
+                response_body = await t(dest.read_bytes)()
             else:
-                response, text = await make_request()
+                response, response_body = await make_request()
                 if response.ok:
-                    await t(dest.write_text)(text, encoding='utf-8')
+                    await t(dest.write_bytes)(response_body)
 
-        yield _ResponseWrapper(response, text)
+        yield _ResponseWrapper(response, response_body)
 
     def get(
         self,
@@ -199,7 +203,7 @@ def _load_certifi_certs():
 def init_web_client(**kwargs: Any) -> _deferred_types.aiohttp.ClientSession:
     from aiohttp import ClientSession, ClientTimeout, TCPConnector
 
-    make_connector = partial(TCPConnector, force_close=True, limit_per_host=10)
+    make_connector = partial(TCPConnector, limit_per_host=10)
     certifi_certs = _load_certifi_certs()
     if certifi_certs:
         import ssl
@@ -219,20 +223,13 @@ def init_web_client(**kwargs: Any) -> _deferred_types.aiohttp.ClientSession:
 
 
 @object.__new__
-class _DummyResolver(BaseResolver):
-    strategies = frozenset()
-
+class _DummyResolver:
     async def resolve(
         self, defns: Sequence[Defn]
     ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
         return dict.fromkeys(defns, R.PkgSourceInvalid())
 
 
-class _ResolverDict(dict):  # type: ignore
-    def __missing__(self, key: str) -> Resolver:
-        return _DummyResolver
-
-
 async def capture_manager_exc_async(
     awaitable: Awaitable[_T],
 ) -> _T | R.ManagerError | R.InternalError:
@@ -284,20 +281,6 @@ _dummy_locks: LocksType = defaultdict(_DummyLock)
 _locks: cv.ContextVar[LocksType] = cv.ContextVar('_locks', default=_dummy_locks)
 
 
-def prepare_database(config: Config) -> sa_future.Engine:
-    "Connect to and optionally create or migrate the database from a configuration object."
-    engine = sa.create_engine(
-        f'sqlite:///{config.db_file}',
-        # We wanna be able to operate on the database from executor threads
-        # when performing disk I/O
-        connect_args={'check_same_thread': False},
-        # echo=True,
-        future=True,
-    )
-    db.migrate_database(engine, DB_REVISION)
-    return engine
-
-
 def contextualise(
     *,
     web_client: _deferred_types.aiohttp.ClientSession | None = None,
@@ -311,7 +294,7 @@ def contextualise(
 
 
 class Manager:
-    RESOLVERS: list[type[Resolver]] = [
+    RESOLVERS: Sequence[type[Resolver]] = [
         GithubResolver,
         CfCoreResolver,
         WowiResolver,
@@ -322,9 +305,9 @@ class Manager:
 
     _base_catalogue_url = (
         f'https://raw.githubusercontent.com/layday/instawow-data/data/'
-        f'base-catalogue-v{BaseCatalogue.construct().version}.compact.json'
+        f'base-catalogue-v{fields(BaseCatalogue).version.default}.compact.json'
     )
-    _catalogue_filename = f'catalogue-v{Catalogue.construct().version}.json'
+    _catalogue_filename = f'catalogue-v{fields(Catalogue).version.default}.json'
 
     _normalise_search_terms = staticmethod(normalise_names(''))
 
@@ -336,18 +319,15 @@ class Manager:
         self.config: Config = config
         self.database: sa_future.Connection = database
 
-        base_resolver_classes = self.RESOLVERS
+        builtin_resolver_classes = list(self.RESOLVERS)
         if self.config.global_config.access_tokens.cfcore is None:
-            base_resolver_classes = base_resolver_classes[:]
-            base_resolver_classes[base_resolver_classes.index(CfCoreResolver)] = CurseResolver
+            builtin_resolver_classes.remove(CfCoreResolver)
 
         plugin_hook = load_plugins()
-        resolver_classes: Iterable[type[Resolver]] = chain(
-            (r for g in plugin_hook.instawow_add_resolvers() for r in g), base_resolver_classes
-        )
-        self.resolvers: dict[str, Resolver] = _ResolverDict(
-            (r.source, r(self)) for r in resolver_classes
+        resolver_classes = chain(
+            (r for g in plugin_hook.instawow_add_resolvers() for r in g), builtin_resolver_classes
         )
+        self.resolvers: Mapping[str, Resolver] = {r.metadata.id: r(self) for r in resolver_classes}
 
         self.web_client: _CacheFauxClientSession = _CacheFauxClientSession(
             self.config.global_config.cache_dir
@@ -358,7 +338,7 @@ class Manager:
     @classmethod
     def from_config(cls, config: Config) -> tuple[Manager, Callable[[], None]]:
         "Instantiate the manager from a configuration object."
-        db_conn = prepare_database(config).connect()
+        db_conn = db.prepare_database(config.db_uri, DB_REVISION).connect()
         return (cls(config, db_conn), db_conn.close)
 
     @property
@@ -370,12 +350,12 @@ class Manager:
         "Attempt to extract the definition source and alias from a URI."
 
         def from_urn():
-            source, alias = value.partition(':')[::2]
+            source, _, alias = value.partition(':')
             if alias:
                 yield (source, alias)
 
         aliases_from_url = (
-            (r.source, a)
+            (r.metadata.id, a)
             for r in self.resolvers.values()
             for a in (r.get_alias_from_url(URL(value)),)
             if a
@@ -388,7 +368,7 @@ class Manager:
             self.database.execute(
                 sa.select(sa.text('1'))
                 .select_from(db.pkg)
-                .filter(
+                .where(
                     db.pkg.c.source == defn.source,
                     (db.pkg.c.id == defn.alias)
                     | (db.pkg.c.id == defn.id)
@@ -402,7 +382,7 @@ class Manager:
         "Retrieve a package from the database."
         maybe_row_mapping = (
             self.database.execute(
-                sa.select(db.pkg).filter(
+                sa.select(db.pkg).where(
                     db.pkg.c.source == defn.source,
                     (db.pkg.c.id == defn.alias)
                     | (db.pkg.c.id == defn.id)
@@ -416,7 +396,7 @@ class Manager:
             maybe_row_mapping = (
                 self.database.execute(
                     sa.select(db.pkg)
-                    .filter(db.pkg.c.slug.contains(defn.alias))
+                    .where(db.pkg.c.slug.contains(defn.alias))
                     .order_by(db.pkg.c.name)
                 )
                 .mappings()
@@ -432,7 +412,7 @@ class Manager:
                 sa.select(db.pkg)
                 .distinct()
                 .join(db.pkg_folder)
-                .filter(db.pkg_folder.c.name.in_(top_level_folders))
+                .where(db.pkg_folder.c.name.in_(top_level_folders))
             ).all()
             if installed_conflicts:
                 raise R.PkgConflictsWithInstalled(installed_conflicts)
@@ -452,7 +432,7 @@ class Manager:
 
             extract(self.config.addon_dir)
 
-        pkg = evolve_model_obj(pkg, folders=[{'name': f} for f in sorted(top_level_folders)])
+        pkg = evolve(pkg, folders=[PkgFolder(name=f) for f in sorted(top_level_folders)])
         pkg.insert(self.database)
         return R.PkgInstalled(pkg)
 
@@ -463,7 +443,7 @@ class Manager:
                 sa.select(db.pkg)
                 .distinct()
                 .join(db.pkg_folder)
-                .filter(
+                .where(
                     db.pkg_folder.c.pkg_source != new_pkg.source,
                     db.pkg_folder.c.pkg_id != new_pkg.id,
                     db.pkg_folder.c.name.in_(top_level_folders),
@@ -487,9 +467,7 @@ class Manager:
 
             extract(self.config.addon_dir)
 
-        new_pkg = evolve_model_obj(
-            new_pkg, folders=[{'name': f} for f in sorted(top_level_folders)]
-        )
+        new_pkg = evolve(new_pkg, folders=[PkgFolder(name=f) for f in sorted(top_level_folders)])
         new_pkg.insert(self.database)
         return R.PkgUpdated(old_pkg, new_pkg)
 
@@ -514,7 +492,9 @@ class Manager:
                 raw_catalogue = await t(catalogue_json.read_bytes)()
                 start = time.perf_counter()
                 # Skip validation when loading the catalogue from cache.
-                self._catalogue = Catalogue.from_cache(raw_catalogue)
+                self._catalogue = catalogue_converter.structure(
+                    json.loads(raw_catalogue), Catalogue
+                )
                 logger.debug(f'loaded catalogue from cache in {time.perf_counter() - start:.3f}s')
         else:
             async with self.web_client.get(
@@ -527,7 +507,14 @@ class Manager:
                 raw_catalogue = await response.json(content_type=None)
 
             self._catalogue = Catalogue.from_base_catalogue(raw_catalogue, None)
-            await t(catalogue_json.write_text)(self._catalogue.json(), encoding='utf-8')
+            await t(catalogue_json.write_text)(
+                json.dumps(
+                    catalogue_converter.unstructure(  # pyright: ignore[reportUnknownMemberType]
+                        self._catalogue
+                    )
+                ),
+                encoding='utf-8',
+            )
 
         return self._catalogue
 
@@ -560,10 +547,10 @@ class Manager:
                 if d.source != pkg.source
             )
 
-        resolver_sources = list(self.resolvers.keys())
+        source_priority = {n: i for i, n in enumerate(self.resolvers)}
 
         return {
-            p: sorted(d, key=lambda d: resolver_sources.index(d.source))
+            p: sorted(d, key=lambda d: source_priority.get(d.source, float('inf')))
             for p in pkgs
             for d in (get_catalogue_defns(p) | await t(extract_addon_toc_defns)(p),)
             if d
@@ -589,8 +576,7 @@ class Manager:
 
         deps = await self.resolve(list(starmap(Defn, dep_defns)))
         pretty_deps = {
-            evolve_model_obj(d, alias=r.slug) if isinstance(r, models.Pkg) else d: r
-            for d, r in deps.items()
+            evolve(d, alias=r.slug) if isinstance(r, models.Pkg) else d: r for d, r in deps.items()
         }
         return pretty_deps
 
@@ -603,7 +589,7 @@ class Manager:
 
         defns_by_source = bucketise(defns, key=lambda v: v.source)
         results = await gather(
-            (self.resolvers[s].resolve(b) for s, b in defns_by_source.items()),
+            (self.resolvers.get(s, _DummyResolver).resolve(b) for s, b in defns_by_source.items()),
             capture_manager_exc_async,
         )
         results_by_defn: dict[Defn, Any] = chain_dict(
@@ -618,18 +604,11 @@ class Manager:
             results_by_defn.update(await self._resolve_deps(results_by_defn.values()))
         return results_by_defn
 
-    async def get_changelog(self, uri: str) -> str:
+    async def get_changelog(self, source: str, uri: str) -> str:
         "Retrieve a changelog from a URI."
-        url = URL(uri)
-        if url.scheme == 'data' and url.raw_path.startswith(','):
-            return urllib.parse.unquote(url.raw_path[1:])
-        elif url.scheme in {'http', 'https'}:
-            async with self.web_client.get(url, {'days': 1}, raise_for_status=True) as response:
-                return await response.text()
-        elif url.scheme == 'file':
-            return await t(Path(file_uri_to_path(uri)).read_text)(encoding='utf-8')
-        else:
-            raise ValueError('Unsupported URI with scheme', url.scheme)
+        if source not in self.resolvers:
+            raise R.PkgSourceInvalid
+        return await self.resolvers[source].get_changelog(URL(uri))
 
     async def search(
         self,
@@ -640,7 +619,8 @@ class Manager:
         installed_only: bool = False,
     ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
-        import rapidfuzz
+        import rapidfuzz.fuzz
+        import rapidfuzz.process
 
         catalogue = await self.synchronise()
 
@@ -787,7 +767,7 @@ class Manager:
             # corresponding installed package.  Using the ID has the benefit
             # of resolving installed-but-renamed packages - the slug is
             # transient but the ID isn't
-            evolve_model_obj(d, id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
+            evolve(d, id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
             for d, p in defns_to_pkgs.items()
         }
         # Discard the reconstructed ``Defn``s
@@ -828,7 +808,7 @@ class Manager:
     @_with_lock('change state')
     async def remove(
         self, defns: Sequence[Defn], keep_folders: bool
-    ) -> dict[Defn, R.PkgRemoved | R.ManagerError | R.InternalError]:
+    ) -> dict[Defn, R.PkgRemoved | R.PkgNotInstalled | R.ManagerError | R.InternalError]:
         "Remove packages by their definition."
         results = chain_dict(
             defns,
@@ -869,7 +849,7 @@ class Manager:
             if not pkg:
                 raise R.PkgNotInstalled
 
-            elif {defn.strategy} <= strategies <= self.resolvers[pkg.source].strategies:
+            elif {defn.strategy} <= strategies <= self.resolvers[pkg.source].metadata.strategies:
                 self.database.execute(
                     sa.update(db.pkg_options)
                     .filter_by(pkg_source=pkg.source, pkg_id=pkg.id)
