@@ -5,11 +5,15 @@ from functools import lru_cache
 from typing import Any
 
 import click
-import pluggy
+import pluggy as _pluggy  # pyright: ignore
+from typing_extensions import Protocol
 
 from . import resolvers
 
+pluggy: Any = _pluggy
+
 _project_name = __package__
+_entry_point = f'{_project_name}.plugins'
 
 hookspec = pluggy.HookspecMarker(_project_name)
 hookimpl = pluggy.HookimplMarker(_project_name)
@@ -24,14 +28,22 @@ class InstawowPlugin:
         ...
 
     @hookspec
-    def instawow_add_resolvers(self) -> Iterable[resolvers.Resolver]:
+    def instawow_add_resolvers(self) -> Iterable[type[resolvers.Resolver]]:
         "Additional resolvers to load."
         ...
 
 
+class _InstawowPluginHookRelay(Protocol):
+    def instawow_add_commands(self) -> Iterable[Iterable[click.Command]]:
+        ...
+
+    def instawow_add_resolvers(self) -> Iterable[Iterable[type[resolvers.Resolver]]]:
+        ...
+
+
 @lru_cache(maxsize=None)
-def load_plugins() -> Any:
+def load_plugins() -> _InstawowPluginHookRelay:
     plugin_manager = pluggy.PluginManager(_project_name)
     plugin_manager.add_hookspecs(InstawowPlugin)
-    plugin_manager.load_setuptools_entrypoints(f'{_project_name}.plugins')
+    plugin_manager.load_setuptools_entrypoints(_entry_point)
     return plugin_manager.hook
