@@ -1,12 +1,14 @@
-# pyright: reportUnusedFunction=false
-
 from __future__ import annotations
 
 import asyncio
 from collections.abc import Sequence
 from functools import partial
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
+from attr import attrib as attrs_attrib
+import attrs
+import cattrs
+from exceptiongroup import ExceptionGroup
 from prompt_toolkit.application import Application
 from prompt_toolkit.document import Document
 from prompt_toolkit.formatted_text.html import HTML
@@ -15,30 +17,49 @@ from prompt_toolkit.keys import Keys
 from prompt_toolkit.shortcuts.progress_bar import ProgressBar, formatters
 from prompt_toolkit.styles import Style
 from prompt_toolkit.validation import ValidationError, Validator
-import pydantic
-from questionary import Choice
-from questionary import confirm as _confirm
-from questionary import path as _path
+from questionary import Choice, confirm as _confirm, password as _password, path as _path
 from questionary.prompts.common import InquirerControl, Separator, create_inquirer_layout
 from questionary.question import Question
 
 from . import models
 
+if TYPE_CHECKING:
+    ExceptionGroup = ExceptionGroup[Exception]
 
-class PydanticValidator(Validator):
-    "One-off validators for Pydantic model fields."
 
-    def __init__(self, model: type[pydantic.BaseModel], field: str) -> None:
-        self.model = model
-        self.field = field
+class AttrFieldValidator(Validator):
+    "One-off validators for attrs fields."
+
+    def __init__(self, attribute: attrs.Attribute[object], converter: cattrs.GenConverter) -> None:
+        self._field_name = attribute.name
+        self._FieldWrapper = attrs.make_class(
+            '_FieldWrapper',
+            {
+                self._field_name: attrs_attrib(
+                    default=attribute.default,
+                    validator=attribute.validator,
+                    repr=attribute.repr,
+                    hash=attribute.hash,
+                    init=attribute.init,
+                    metadata=attribute.metadata,
+                    type=attribute.type,
+                    converter=attribute.converter,
+                    kw_only=attribute.kw_only,
+                    eq=attribute.eq,
+                    order=attribute.order,
+                    on_setattr=attribute.on_setattr,
+                )
+            },
+        )
+        self._converter = converter
 
     def validate(self, document: Document) -> None:
         try:
-            self.model.parse_obj({self.field: document.text})
-        except pydantic.ValidationError as error:
-            error_at_loc = next((e for e in error.errors() if e['loc'] == (self.field,)), None)
-            if error_at_loc:
-                raise ValidationError(0, error_at_loc['msg'])
+            self._converter.structure({self._field_name: document.text}, self._FieldWrapper)
+        except ExceptionGroup as exc_group:
+            raise ValidationError(0, '\n'.join(map(str, exc_group.exceptions)))
+        except Exception as exc:
+            raise ValidationError(0, str(exc))
 
 
 class PkgChoice(Choice):
@@ -59,10 +80,11 @@ qstyle = Style(
 
 
 SKIP = ()
-skip = Choice([('', 'skip')], SKIP)
+skip = Choice([('underline', 's'), ('', 'kip')], SKIP)
 
 confirm = partial(_confirm, qmark='?', style=qstyle)
 path = partial(_path, qmark='>', style=qstyle)
+password = partial(_password, style=qstyle)
 
 
 def checkbox(message: str, choices: Sequence[Choice], **inquirer_kwargs: Any) -> Question:
@@ -86,11 +108,11 @@ def checkbox(message: str, choices: Sequence[Choice], **inquirer_kwargs: Any) ->
 
     @bindings.add(Keys.ControlQ, eager=True)
     @bindings.add(Keys.ControlC, eager=True)
-    def abort(event: KeyPressEvent):
+    def abort(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         event.app.exit(exception=KeyboardInterrupt, style='class:aborting')
 
     @bindings.add(' ', eager=True)
-    def toggle(event: KeyPressEvent):
+    def toggle(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         pointed_choice = ic.get_pointed_at().value
         if pointed_choice in ic.selected_options:
             ic.selected_options.remove(pointed_choice)
@@ -98,7 +120,7 @@ def checkbox(message: str, choices: Sequence[Choice], **inquirer_kwargs: Any) ->
             ic.selected_options.append(pointed_choice)
 
     @bindings.add('i', eager=True)
-    def invert(event: KeyPressEvent):
+    def invert(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         inverted_selection = [
             c.value
             for c in ic.choices
@@ -110,25 +132,25 @@ def checkbox(message: str, choices: Sequence[Choice], **inquirer_kwargs: Any) ->
 
     @bindings.add(Keys.Down, eager=True)
     @bindings.add('j', eager=True)
-    def move_cursor_down(event: KeyPressEvent):
+    def move_cursor_down(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.select_next()
         while not ic.is_selection_valid():
             ic.select_next()
 
     @bindings.add(Keys.Up, eager=True)
     @bindings.add('k', eager=True)
-    def move_cursor_up(event: KeyPressEvent):
+    def move_cursor_up(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.select_previous()
         while not ic.is_selection_valid():
             ic.select_previous()
 
     @bindings.add(Keys.ControlM, eager=True)
-    def set_answer(event: KeyPressEvent):
+    def set_answer(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.is_answered = True
         event.app.exit(result=[c.value for c in ic.get_selected_values()])
 
     @bindings.add('o', eager=True)
-    def open_url(event: KeyPressEvent):
+    def open_url(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         choice = ic.get_pointed_at()
         if isinstance(choice, PkgChoice):
             import webbrowser
@@ -136,7 +158,7 @@ def checkbox(message: str, choices: Sequence[Choice], **inquirer_kwargs: Any) ->
             webbrowser.open(choice.pkg.url)
 
     @bindings.add(Keys.Any)
-    def default(event: KeyPressEvent):
+    def default(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         # Disallow inserting other text
         pass
 
@@ -176,30 +198,30 @@ def select(
 
     @bindings.add(Keys.ControlQ, eager=True)
     @bindings.add(Keys.ControlC, eager=True)
-    def abort(event: KeyPressEvent):
+    def abort(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         event.app.exit(exception=KeyboardInterrupt, style='class:aborting')
 
     @bindings.add(Keys.Down, eager=True)
     @bindings.add('j', eager=True)
-    def move_cursor_down(event: KeyPressEvent):
+    def move_cursor_down(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.select_next()
         while not ic.is_selection_valid():
             ic.select_next()
 
     @bindings.add(Keys.Up, eager=True)
     @bindings.add('k', eager=True)
-    def move_cursor_up(event: KeyPressEvent):
+    def move_cursor_up(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.select_previous()
         while not ic.is_selection_valid():
             ic.select_previous()
 
     @bindings.add(Keys.ControlM, eager=True)
-    def set_answer(event: KeyPressEvent):
+    def set_answer(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         ic.is_answered = True
         event.app.exit(result=ic.get_pointed_at().value)
 
     @bindings.add('o', eager=True)
-    def open_url(event: KeyPressEvent):
+    def open_url(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         choice = ic.get_pointed_at()
         if isinstance(choice, PkgChoice):
             import webbrowser
@@ -209,12 +231,12 @@ def select(
     if skip in ic.choices:
 
         @bindings.add('s', eager=True)
-        def skip_question(event: KeyPressEvent):
+        def skip_question(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
             ic.pointed_at = -1
             set_answer(event)
 
     @bindings.add(Keys.Any)
-    def default(event: KeyPressEvent):
+    def default(event: KeyPressEvent):  # pyright: ignore[reportUnusedFunction]
         # Disallow inserting other text
         pass
 
