@@ -6,21 +6,20 @@ from collections.abc import Awaitable, Callable, Iterable, Iterator, Mapping, Se
 from datetime import datetime, timedelta
 import enum
 from functools import partial, wraps
-from itertools import chain, groupby, repeat
+from itertools import chain, groupby, islice, repeat
 import os
 from pathlib import Path, PurePath
 import posixpath
 from shutil import move as _move
+import sys
 from tempfile import mkdtemp
 from typing import Any, Generic, Hashable, TypeVar, overload
 
-import pydantic
 from typing_extensions import ParamSpec
 
 _T = TypeVar('_T')
 _U = TypeVar('_U')
 _THashable = TypeVar('_THashable', bound=Hashable)
-_TBaseModel = TypeVar('_TBaseModel', bound=pydantic.BaseModel)
 _P = ParamSpec('_P')
 
 
@@ -71,12 +70,17 @@ class cached_property(Generic[_T, _U]):
             return v
 
 
-def bucketise(iterable: Iterable[_U], key: Callable[[_U], _T]) -> defaultdict[_T, list[_U]]:
+def fill(it: Iterable[_T], fill: _T, number: int) -> Iterable[_T]:
+    "Append entries to a sized iterable."
+    return islice(chain(it, repeat(fill)), 0, number)
+
+
+def bucketise(iterable: Iterable[_U], key: Callable[[_U], _T]) -> dict[_T, list[_U]]:
     "Place the elements of an iterable in a bucket according to ``key``."
     bucket: defaultdict[_T, list[_U]] = defaultdict(list)
     for value in iterable:
         bucket[key(value)].append(value)
-    return bucket
+    return dict(bucket)
 
 
 def chain_dict(
@@ -113,14 +117,14 @@ def merge_intersecting_sets(it: Iterable[frozenset[_T]]) -> Iterator[frozenset[_
 
 
 @overload
-async def gather(it: Iterable[Awaitable[_U]], wrapper: None = ...) -> list[_U]:
+async def gather(it: Iterable[Awaitable[_U]], wrapper: None = None) -> list[_U]:
     ...
 
 
 @overload
 async def gather(
     it: Iterable[Awaitable[_U]],
-    wrapper: Callable[[Awaitable[_U]], Awaitable[_T]] = ...,
+    wrapper: Callable[[Awaitable[_U]], Awaitable[_T]],
 ) -> list[_T]:
     ...
 
@@ -213,13 +217,13 @@ def is_not_stale(path: Path, ttl: Mapping[str, float]) -> bool:
     )
 
 
-def find_addon_zip_base_dirs(names: Sequence[str]) -> Iterator[str]:
+def find_addon_zip_tocs(names: Iterable[str]) -> Iterator[tuple[str, str]]:
     "Find top-level folders in a list of ZIP member paths."
     for name in names:
         if name.count(posixpath.sep) == 1:
             head, tail = posixpath.split(name)
             if tail.startswith(head) and tail[-4:].lower() == '.toc':
-                yield head
+                yield (name, head)
 
 
 def make_zip_member_filter(base_dirs: Set[str]) -> Callable[[str], bool]:
@@ -245,6 +249,10 @@ def file_uri_to_path(file_uri: str) -> str:
     return unprefixed_path
 
 
+def extract_byte_range_offset(content_range: str):
+    return int(content_range.replace('bytes ', '').partition('-')[0])
+
+
 def normalise_names(replace_delim: str) -> Callable[[str], str]:
     import string
 
@@ -256,5 +264,10 @@ def normalise_names(replace_delim: str) -> Callable[[str], str]:
     return normalise
 
 
-def evolve_model_obj(model_obj: _TBaseModel, **kwargs: object) -> _TBaseModel:
-    return model_obj.__class__.parse_obj({**model_obj.__dict__, **kwargs})
+def reveal_folder(path: str | os.PathLike[str]) -> None:
+    if sys.platform == 'win32':
+        os.startfile(path, 'explore')
+    else:
+        import click
+
+        click.launch(os.fspath(path), locate=True)
