@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import re
 
+from attrs import evolve
 import pytest
 from yarl import URL
 
@@ -10,14 +11,13 @@ from instawow.common import Flavour, Strategy
 from instawow.manager import Manager
 from instawow.models import Pkg
 from instawow.resolvers import (
-    CurseResolver,
+    CfCoreResolver,
     Defn,
     GithubResolver,
     Resolver,
     TukuiResolver,
     WowiResolver,
 )
-from instawow.utils import evolve_model_obj
 
 
 @pytest.mark.parametrize('strategy', [Strategy.default, Strategy.latest])
@@ -29,16 +29,17 @@ async def test_curse_simple_strategies(iw_manager: Manager, strategy: Strategy):
     results = await iw_manager.resolve([flavourful, retail_only, classic_only])
 
     if iw_manager.config.game_flavour is Flavour.vanilla_classic:
-        assert results[flavourful].version.endswith('Release-classic')
-        assert (
-            type(results[retail_only]) is R.PkgFileUnavailable
-            and results[retail_only].message
-            == f"no files matching vanilla_classic using {strategy} strategy"
-        )
+        if strategy is Strategy.default:
+            assert (
+                type(results[retail_only]) is R.PkgFileUnavailable
+                and results[retail_only].message
+                == f"no files matching vanilla_classic using {strategy} strategy"
+            )
+        else:
+            assert type(results[retail_only]) is Pkg
         assert type(results[classic_only]) is Pkg
 
     elif iw_manager.config.game_flavour is Flavour.burning_crusade_classic:
-        assert results[flavourful].version.endswith('Release-bcc')
         assert (
             type(results[retail_only]) is R.PkgFileUnavailable
             and results[retail_only].message
@@ -47,7 +48,6 @@ async def test_curse_simple_strategies(iw_manager: Manager, strategy: Strategy):
         assert type(results[classic_only]) is Pkg
 
     elif iw_manager.config.game_flavour is Flavour.retail:
-        assert results[flavourful].version.endswith('Release')
         assert type(results[retail_only]) is Pkg
         assert (
             type(results[classic_only]) is R.PkgFileUnavailable
@@ -69,11 +69,11 @@ async def test_curse_any_flavour_strategy(iw_manager: Manager):
 
 
 async def test_curse_version_pinning(iw_manager: Manager):
-    defn = Defn('curse', 'molinari').with_version('70300.51-Release')
+    defn = Defn('curse', 'molinari').with_version('80000.58-Release')
     results = await iw_manager.resolve([defn])
     assert (
         results[defn].options.strategy == Strategy.version
-        and results[defn].version == '70300.51-Release'
+        and results[defn].version == '80000.58-Release'
     )
 
 
@@ -94,7 +94,7 @@ async def test_curse_changelog_is_url(iw_manager: Manager):
 
     results = await iw_manager.resolve([molinari])
     assert re.match(
-        r'https://addons-ecs\.forgesvc\.net/api/v2/addon/\d+/file/\d+/changelog',
+        r'https://api\.curseforge\.com/v1/mods/\d+/files/\d+/changelog',
         results[molinari].changelog_url,
     )
 
@@ -149,7 +149,7 @@ async def test_tukui_changelog_url_for_addon_type(iw_manager: Manager):
     results = await iw_manager.resolve([ui_suite, regular_addon])
 
     if iw_manager.config.game_flavour is Flavour.retail:
-        assert results[ui_suite].changelog_url == 'https://www.tukui.org/ui/tukui/changelog#20.23'
+        assert results[ui_suite].changelog_url == 'https://www.tukui.org/ui/tukui/changelog#20.27'
     assert results[regular_addon].changelog_url.startswith('data:,')
 
 
@@ -184,9 +184,9 @@ async def test_github_changelog_is_data_url(iw_manager: Manager):
 
 @pytest.mark.parametrize('resolver', Manager.RESOLVERS)
 async def test_unsupported_strategies(iw_manager: Manager, resolver: Resolver):
-    defn = Defn(resolver.source, 'foo')
-    for strategy in set(Strategy) - {Strategy.version} - resolver.strategies:
-        strategy_defn = evolve_model_obj(defn, strategy=strategy)
+    defn = Defn(resolver.metadata.id, 'foo')
+    for strategy in set(Strategy) - {Strategy.version} - resolver.metadata.strategies:
+        strategy_defn = evolve(defn, strategy=strategy)
 
         results = await iw_manager.resolve([strategy_defn])
 
@@ -199,8 +199,8 @@ async def test_unsupported_strategies(iw_manager: Manager, resolver: Resolver):
 @pytest.mark.parametrize(
     ('resolver', 'url', 'extracted_alias'),
     [
-        (CurseResolver, 'https://www.curseforge.com/wow/addons/molinari', 'molinari'),
-        (CurseResolver, 'https://www.curseforge.com/wow/addons/molinari/download', 'molinari'),
+        (CfCoreResolver, 'https://www.curseforge.com/wow/addons/molinari', 'molinari'),
+        (CfCoreResolver, 'https://www.curseforge.com/wow/addons/molinari/download', 'molinari'),
         (WowiResolver, 'https://www.wowinterface.com/downloads/landing.php?fileid=13188', '13188'),
         (WowiResolver, 'https://wowinterface.com/downloads/landing.php?fileid=13188', '13188'),
         (WowiResolver, 'https://www.wowinterface.com/downloads/fileinfo.php?id=13188', '13188'),
