@@ -18,15 +18,12 @@ from instawow import __version__
 from instawow.cli import cli
 from instawow.common import Flavour
 from instawow.config import Config
-from instawow.models import PkgList
 
 
 @pytest.fixture
 def feed_pt():
-    pipe_input = create_pipe_input()
-    with create_app_session(input=pipe_input, output=DummyOutput()):
-        yield pipe_input.send_text
-    pipe_input.close()
+    with create_pipe_input() as input_, create_app_session(input=input_, output=DummyOutput()):
+        yield input_.send_text
 
 
 @pytest.fixture
@@ -65,20 +62,14 @@ def pretend_install_molinari_and_run(
     yield run
 
 
-@pytest.fixture
-def molinari_version_suffix(
-    iw_config: Config,
-):
-    if iw_config.game_flavour is Flavour.vanilla_classic:
-        return '-classic'
-    if iw_config.game_flavour is Flavour.burning_crusade_classic:
-        return '-bcc'
-    else:
-        return ''
-
-
 @pytest.mark.parametrize(
-    'alias', ['curse:molinari', 'wowi:13188-molinari', 'tukui:1', 'github:p3lim-wow/Molinari']
+    'alias',
+    [
+        'curse:molinari',
+        'wowi:13188-molinari',
+        'tukui:1',
+        'github:p3lim-wow/Molinari',
+    ],
 )
 def test_valid_pkg_lifecycle(
     run: C[[str], Result],
@@ -152,7 +143,7 @@ def test_version_strategy_lifecycle(
     run: C[[str], Result],
 ):
     assert run('install curse:molinari').output.startswith(
-        '✓ curse:molinari\n  installed 90105.81-Release'
+        '✓ curse:molinari\n  installed 90200.82-Release'
     )
     assert (
         run('install --version foo curse:molinari').output
@@ -175,7 +166,6 @@ def test_version_strategy_lifecycle(
 
 def test_install_options(
     run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     assert run(
         'install'
@@ -185,7 +175,7 @@ def test_install_options(
     ).output == dedent(
         f'''\
         ✓ curse:molinari
-          installed 90105.81-Release{molinari_version_suffix}
+          installed 90200.82-Release
         ✗ curse:molinari
           package folders conflict with installed package Molinari
             (curse:20338)
@@ -220,14 +210,13 @@ def test_install_option_order_is_respected(
 
 def test_install_argument_is_not_required(
     run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     assert run(
         'install -s latest curse:molinari --version 80000.57-Release curse:molinari'
     ).output == dedent(
         f'''\
         ✓ curse:molinari
-          installed 90105.81-Release{molinari_version_suffix}
+          installed 90200.82-Release
         ✗ curse:molinari
           package folders conflict with installed package Molinari
             (curse:20338)
@@ -235,20 +224,30 @@ def test_install_argument_is_not_required(
     )
 
 
+def test_install_invalid_defn(
+    run: C[[str], Result],
+):
+    result = run('install foo')
+    assert result.exit_code == 2
+    assert result.output.endswith("Error: Invalid value for '[ADDONS]...': foo\n")
+
+
 def test_configure__show_active_profile(
     iw_config: Config,
     run: C[[str], Result],
 ):
-    assert run('configure --show-active').output == iw_config.json(indent=2) + '\n'
+    assert run('configure --show-active').output == iw_config.encode_for_display() + '\n'
 
 
+@pytest.mark.parametrize('command', ['configure', 'list'], ids=['explicit', 'implicit'])
 def test_configure__create_new_profile(
     feed_pt: C[[str], None],
     iw_config: Config,
     run: C[[str], Result],
+    command: str,
 ):
     feed_pt(f'{iw_config.addon_dir}\r\rY\r')
-    assert run('-p foo configure').output == (
+    assert run(f'-p foo {command}').output == (
         'Navigate to https://github.com/login/device and paste the code below:\n'
         '  WDJB-MJHT\n'
         'Waiting...\n'
@@ -306,7 +305,6 @@ def test_rollback__single_version(
 def test_rollback__multiple_versions(
     feed_pt: C[[str], None],
     run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     assert run('install --version 80000.57-Release curse:molinari').exit_code == 0
     assert run('remove curse:molinari').exit_code == 0
@@ -315,14 +313,13 @@ def test_rollback__multiple_versions(
     assert run('rollback curse:molinari').output == dedent(
         f'''\
         ✓ curse:molinari
-          updated 90105.81-Release{molinari_version_suffix} to 80000.57-Release
+          updated 90200.82-Release to 80000.57-Release
         '''
     )
 
 
 def test_rollback__multiple_versions_promptless(
     run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     assert run('install --version 80000.57-Release curse:molinari').exit_code == 0
     assert run('remove curse:molinari').exit_code == 0
@@ -330,7 +327,7 @@ def test_rollback__multiple_versions_promptless(
     assert run('rollback --version 80000.57-Release curse:molinari').output == dedent(
         f'''\
         ✓ curse:molinari
-          updated 90105.81-Release{molinari_version_suffix} to 80000.57-Release
+          updated 90200.82-Release to 80000.57-Release
         '''
     )
 
@@ -339,7 +336,6 @@ def test_rollback__multiple_versions_promptless(
 def test_rollback__rollback_multiple_versions(
     feed_pt: C[[str], None],
     run: C[[str], Result],
-    molinari_version_suffix: str,
     options: str,
 ):
     assert run('install curse:molinari').exit_code == 0
@@ -349,7 +345,7 @@ def test_rollback__rollback_multiple_versions(
     assert run(f'rollback {options} curse:molinari').output == dedent(
         f'''\
         ✓ curse:molinari
-          updated 80000.57-Release to 90105.81-Release{molinari_version_suffix}
+          updated 80000.57-Release to 90200.82-Release
         '''
     )
 
@@ -374,14 +370,27 @@ def test_reconcile__list_unreconciled(
     )
 
 
+def test_reconcile_leftovers(
+    feed_pt: C[[str], None],
+    pretend_install_molinari_and_run: C[[str], Result],
+):
+    feed_pt('sss')  # Skip
+    assert pretend_install_molinari_and_run('reconcile').output.endswith(
+        # fmt: off
+        'unreconciled\n'
+        '------------\n'
+        'Molinari    \n'
+        # fmt: on
+    )
+
+
 def test_reconcile__auto_reconcile(
     pretend_install_molinari_and_run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     assert pretend_install_molinari_and_run('reconcile --auto').output == dedent(
         f'''\
         ✓ curse:molinari
-          installed 90105.81-Release{molinari_version_suffix}
+          installed 90200.82-Release
         '''
     )
 
@@ -397,14 +406,13 @@ def test_reconcile__abort_interactive_reconciliation(
 def test_reconcile__complete_interactive_reconciliation(
     feed_pt: C[[str], None],
     pretend_install_molinari_and_run: C[[str], Result],
-    molinari_version_suffix,
 ):
     feed_pt('\r\r')
     assert pretend_install_molinari_and_run('reconcile').output.endswith(
         dedent(
             f'''\
             ✓ curse:molinari
-              installed 90105.81-Release{molinari_version_suffix}
+              installed 90200.82-Release
             '''
         )
     )
@@ -426,7 +434,7 @@ def test_reconcile__rereconcile(
         ✓ curse:molinari
           removed
         ✓ github:p3lim-wow/molinari
-          installed 90105.81-Release
+          installed 90200.82-Release
         '''
     )
 
@@ -439,7 +447,6 @@ def test_reconcile__cannot_use_auto_with_installed(
     assert 'Cannot use "--auto" with "--installed"' in result.output
 
 
-@pytest.mark.skip
 def test_search__no_results(
     run: C[[str], Result],
 ):
@@ -465,13 +472,12 @@ def test_search__exit_after_selection(
 def test_search__install_one(
     feed_pt: C[[str], None],
     run: C[[str], Result],
-    molinari_version_suffix: str,
 ):
     feed_pt(' \r\r')  # space, enter, enter
     assert run('search molinari --source curse').output == dedent(
         f'''\
         ✓ curse:molinari
-          installed 90105.81-Release{molinari_version_suffix}
+          installed 90200.82-Release
         '''
     )
 
@@ -484,7 +490,7 @@ def test_search__install_multiple_conflicting(
     assert run('search molinari').output == dedent(
         f'''\
         ✓ wowi:13188-molinari
-          installed 90105.81-Release
+          installed 90200.82-Release
         ✗ curse:molinari
           package folders conflict with installed package Molinari
             (wowi:13188)
@@ -496,9 +502,9 @@ def test_changelog_output(
     install_molinari_and_run: C[[str], Result],
 ):
     output = (
-        'Changes in 90105.81-Release:'
+        'Changes in 90200.82-Release:'
         if shutil.which('pandoc')
-        else '<h3>Changes in 90105.81-Release:</h3>'
+        else '<h3>Changes in 90200.82-Release:</h3>'
     )
     assert install_molinari_and_run('view-changelog curse:molinari').output.startswith(output)
 
@@ -508,16 +514,16 @@ def test_changelog_output_no_convert(
 ):
     assert install_molinari_and_run(
         'view-changelog --no-convert curse:molinari'
-    ).output.startswith('<h3>Changes in 90105.81-Release:</h3>')
+    ).output.startswith('<h3>Changes in 90200.82-Release:</h3>')
 
 
 def test_argless_changelog_output(
     install_molinari_and_run: C[[str], Result],
 ):
     output = (
-        'curse:molinari:\n  Changes in 90105.81-Release:'
+        'curse:molinari:\n  Changes in 90200.82-Release:'
         if shutil.which('pandoc')
-        else 'curse:molinari:\n  <h3>Changes in 90105.81-Release:</h3>'
+        else 'curse:molinari:\n  <h3>Changes in 90200.82-Release:</h3>'
     )
     assert install_molinari_and_run('view-changelog').output.startswith(output)
 
@@ -526,7 +532,7 @@ def test_argless_changelog_output_no_convert(
     install_molinari_and_run: C[[str], Result],
 ):
     assert install_molinari_and_run('view-changelog --no-convert').output.startswith(
-        'curse:molinari:\n  <h3>Changes in 90105.81-Release:</h3>'
+        'curse:molinari:\n  <h3>Changes in 90200.82-Release:</h3>'
     )
 
 
@@ -545,7 +551,7 @@ def test_exit_codes_with_substr_match(
     command: str,
     exit_code: int,
 ):
-    monkeypatch.setattr('click.launch', lambda *_, **__: ...)
+    monkeypatch.setattr('instawow.cli.reveal_folder', lambda *_, **__: ...)
     assert install_molinari_and_run(command).exit_code == exit_code
 
 
@@ -563,7 +569,7 @@ def test_json_export(
     install_molinari_and_run: C[[str], Result],
 ):
     output = install_molinari_and_run('list -f json').output
-    assert PkgList.parse_raw(output).__root__[0].name == 'Molinari'
+    assert json.loads(output)[0]['name'] == 'Molinari'
 
 
 def test_show_version(
