@@ -6,7 +6,7 @@ from typing import Any
 
 import pytest
 
-from instawow.common import Flavour
+from instawow.common import Flavour, infer_flavour_from_path
 from instawow.config import Config, GlobalConfig
 
 
@@ -18,10 +18,11 @@ def test_env_vars_have_prio(
     monkeypatch.setenv('INSTAWOW_CONFIG_DIR', '/foo')
     monkeypatch.setenv('INSTAWOW_GAME_FLAVOUR', 'classic')
 
-    global_config = GlobalConfig(**iw_global_config_values)
-    config = Config(global_config=global_config, **iw_config_values)
-
-    assert global_config.config_dir == Path('/foo').resolve()
+    config = Config.from_env(
+        global_config=GlobalConfig.from_env(**iw_global_config_values),
+        **iw_config_values,
+    )
+    assert config.global_config.config_dir == Path('/foo').resolve()
     assert config.game_flavour is Flavour.burning_crusade_classic
 
 
@@ -48,7 +49,7 @@ def test_init_with_nonexistent_addon_dir_raises(
 ):
     global_config = GlobalConfig(**iw_global_config_values).write()
     with pytest.raises(ValueError, match='not a writable directory'):
-        Config(global_config=global_config, **{**iw_config_values, 'addon_dir': 'foo'})
+        Config(global_config=global_config, **{**iw_config_values, 'addon_dir': '#@$foo'})
 
 
 @pytest.mark.skipif(
@@ -86,14 +87,16 @@ def test_default_config_dir_is_unplatform_appropriate(
 
 
 def test_can_infer_flavour_from_path():
-    infer = Config.infer_flavour
-    assert infer('wowzerz/_classic_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer('/foo/bar/_classic_beta_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer('/foo/bar/_classic_ptr_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer('_classic_era_/Interface/AddOns') is Flavour.vanilla_classic
-    assert infer('_classic_era_ptr_/Interface/AddOns') is Flavour.vanilla_classic
-    assert infer('wowzerz/_retail_/Interface/AddOns') is Flavour.retail
-    assert infer('anything goes') is Flavour.retail
+    # fmt: off
+    assert infer_flavour_from_path('wowzerz/_classic_/Interface/AddOns') is Flavour.burning_crusade_classic
+    assert infer_flavour_from_path('/foo/bar/_classic_beta_/Interface/AddOns') is Flavour.burning_crusade_classic
+    assert infer_flavour_from_path('/foo/bar/_classic_ptr_/Interface/AddOns') is Flavour.burning_crusade_classic
+    assert infer_flavour_from_path('_classic_era_/Interface/AddOns') is Flavour.vanilla_classic
+    assert infer_flavour_from_path('_classic_era_beta_/Interface/AddOns') is Flavour.vanilla_classic
+    assert infer_flavour_from_path('_classic_era_ptr_/Interface/AddOns') is Flavour.vanilla_classic
+    assert infer_flavour_from_path('wowzerz/_retail_/Interface/AddOns') is Flavour.retail
+    assert infer_flavour_from_path('anything goes') is Flavour.retail
+    # fmt: on
 
 
 def test_can_list_profiles(
@@ -103,7 +106,7 @@ def test_can_list_profiles(
 ):
     monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_global_config_values['config_dir']))
 
-    global_config = GlobalConfig()
+    global_config = GlobalConfig.read()
     assert global_config.list_profiles() == []
 
     Config(global_config=global_config, **iw_config_values).write()
@@ -120,16 +123,3 @@ def test_can_delete_profile(
     assert config.profile_dir.exists()
     config.delete()
     assert not config.profile_dir.exists()
-
-
-def test_can_omit_global_config(
-    monkeypatch: pytest.MonkeyPatch,
-    iw_global_config_values: dict[str, Any],
-    iw_config_values: dict[str, Any],
-):
-    global_config = GlobalConfig(**iw_global_config_values)
-    Config(global_config=global_config, **iw_config_values).write()
-
-    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_global_config_values['config_dir']))
-    config = Config.read(None, iw_config_values['profile'])
-    assert config.global_config.dict() == global_config.dict()
